==> coventry_0.7.6.orig.tar.gz: coventry-0.7.6/utils/pbx-ping.cpp <==
/*
 * Copyright (C) 2020-2023 David Sugar <tychosoft@gmail.com>.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "compiler.hpp"
#include "ipc.hpp"

#include <args.hpp>
#include <print.hpp>
#include <strings.hpp>
#include <exosip.hpp>

#include <thread>

#include <unistd.h>
#include <pwd.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <netdb.h>
#include <ifaddrs.h>
#include <netinet/in.h>
#include <net/if.h>
#include <arpa/inet.h>

using namespace std::literals::string_literals;
using namespace std::chrono_literals;

namespace {
exosip::context_t context("coventry-utils/" PROJECT_VERSION);
std::string uri = "sip:";

bool ipv6 = false;
bool tls = false;
bool tcp = false;
auto port = 5062;
auto ping_duration = 1000;

args::option opt_ipv6('6',      "--ipv6",   "enable ipv6");
args::option opt_config('c',    "--config", "config file path");
args::option helpflag('h',      "--help",   "display this list");
args::option opt_info('i',      "--info",   "display server info");
args::option opt_proxy('p',     "--proxy",  "proxy route", "uri");
args::option opt_quiet('q',     "--quiet",  "quiet one time");
args::option opt_srv('s',       "--srv",    "enable srv lookups");
args::option opt_tcp('t',       "--tcp",    "enable tcp/tls");
args::option althelp('?',       nullptr, nullptr);

auto to_string(const struct sockaddr_storage *addr) {
    const auto sa = reinterpret_cast<const struct sockaddr *>(addr);
    char buf[256];
    switch(sa->sa_family) {
    case AF_INET:
        inet_ntop(AF_INET, &(((struct sockaddr_in *)sa)->sin_addr), buf, sizeof(buf));  // NOLINT
        return "sip:" + std::string(buf) + ":" + std::to_string(ntohs(((struct sockaddr_in *)sa)->sin_port)); // NOLINT
    case AF_INET6:
        inet_ntop(AF_INET6, &(((struct sockaddr_in6 *)sa)->sin6_addr), buf, sizeof(buf));   // NOLINT
        return "sip:[" + std::string(buf) + "]:" + std::to_string(ntohs(((struct sockaddr_in6 *)sa)->sin6_port));   // NOLINT
;
    default:
        return std::string();
    }
}

auto get_ext(std::string_view text) {
    if(text.size() != 2)
        return 0;

    auto v0 = text[0];
    auto v1 = text[1];

    if(v0 < '1' || v0 > '8')
        return 0;

    if(v1 < '0' || v1 > '9')
        return 0;

    return ((v0 - '0') * 10) + v1 - '0';
}

auto from() {
    char host[256];
    std::string user;
    if(gethostname(host, sizeof(host) - 1))
        throw bad_arg("no hostname set");
    host[sizeof(host) - 1] = 0;
    auto pwd = getpwuid(getuid());
    if(pwd && pwd->pw_name && pwd->pw_name[0])
        user = pwd->pw_name;
    else
        throw bad_arg("no user set");
    endpwent();
    return "sip:" + user + "@" + host + ":" + std::to_string(port);
}

void send_ping(const std::string& uri) {
    osip_message_t *msg = nullptr;
    const exosip::guard_t lock(context);
    eXosip_options_build_request(*context, &msg, uri.c_str(), from().c_str(), *opt_proxy);
    eXosip_options_send_request(*context, msg);
}
} // end namespace

auto main(int argc, const char **argv) -> int {
    using namespace exosip;
    using namespace std::chrono;

    int exit_code = 0;
    try {
        auto count = args::parse(argc, argv);
        if(is(helpflag) || is(althelp) || !count) {
            args::help({"pbx-ping [options] uri|ext"}, {
                "pbx-ping " PROJECT_VERSION,
                "David Sugar <tychosoft@gmail.com>",
                "Ping remove sip endpoint"
            });
            return 0;
        }
        if(count > 1)
            throw bad_arg("only use 1 uri");

        auto uri_arg = args::list()[0];
        ipv6 = *opt_ipv6;
        tcp = *opt_tcp;
        auto ext = get_ext(uri_arg);
        std::string reg_uri;

        if(ext) {
            auto rshm = shm_open(SERVER_IPCPATH ".registry", O_RDONLY, 0660);
            if(rshm < 0)
                throw std::runtime_error("server offline");
            struct stat rino{};
            if(fstat(rshm, &rino) < 0)
                throw std::runtime_error("server offline");

            auto rmax = (rino.st_size - sizeof(pbx_sys)) / sizeof(pbx_reg);
            auto rmem = mmap(nullptr, sizeof(pbx_reg) * rmax + sizeof(pbx_sys), PROT_READ, MAP_SHARED, rshm, 0);
            if(rmem == MAP_FAILED)   // NOLINT
                throw std::runtime_error("server failed");

            const auto& reg = static_cast<pbx_reg*>(rmem);
            const auto& entry = reg[ext - 10];
            time_t expires{};

            time(&expires);
            if(!entry.activated || expires > entry.expires) {
                if(!is(opt_quiet))
                    print(std::cerr, "*** pbx-ping: {}: offline\n", uri_arg);
                ::exit(3);
            }
            reg_uri = to_string(&entry.address);
            uri_arg = reg_uri;
        }

        if(begins_with(uri_arg, "sip:")) {
            tls = false;
            uri = "sip:";
            uri_arg.remove_prefix(4);
        }
        else if(begins_with(uri_arg, "sips:")) {
            tls = true;
            uri = "sips:";
            uri_arg.remove_prefix(5);
        }

        auto pos = uri_arg.find('@');
        auto route = uri;

        uri.append(uri_arg);
        if(pos != std::string::npos)
            route.append(uri_arg.substr(pos));
        else
            route.append(uri_arg);

        if(!is(opt_proxy))
            opt_proxy.set(route.c_str());

        auto type = tcp ? listen_type::tcp : listen_type::udp;
        if(tls)
            type = tcp ? listen_type::tls : listen_type::dtls;

        port = context.find_port(5062, tcp ? IPPROTO_TCP : IPPROTO_UDP);
        context.srv_lookup(is(opt_srv));
        context.set_family(ipv6 ? family_type::ipv6 : family_type::ipv4);
        if(!context.listen("*", type, port))
            throw std::runtime_error("unable to bind to a local port");

        //send_ping(uri); was for exosip2 epoll bug (5.2.0-5.2.1)
        //std::this_thread::sleep_for(std::chrono::milliseconds(20));

        for(;;) {
            send_ping(uri);
            auto start = steady_clock::now();
            auto event = exosip::event_t(context, ping_duration);
            auto reply = duration_cast<milliseconds>(steady_clock::now() - start).count();
            auto response = event.response();
            if(!response) {
                exit_code = 3;
                if(reply < ping_duration)
                    ++exit_code;
                if(is(opt_quiet))
                    return exit_code;
                if(exit_code == 3)
                    print(std::cerr, "*** pbx-ping: {}: unreachable\n", uri);
                else {
                    print(std::cerr, "*** pbx-ping: {}: invalid\n", uri);
                    break;
                }
                if(is(opt_info))
                    break;
                continue;
            }

            auto code = response->status_code;
            auto reason = response->reason_phrase;
            if(code == SIP_OK && is(opt_info)) {
                char *text = nullptr;
                size_t size{};
                osip_message_to_str(response, &text, &size);
                if(text) {
                    print("{}\n", text);
                    release(text);
                }
                return 0;
            }
            if(!is(opt_info) && (code >= 200 && code < 500)) {
                if(is(opt_quiet))
                    return 0;
                print("pbx-ping: {}: ok time={}ms\n", uri, reply);
            }
            else {
                exit_code = 2;
                if(is(opt_quiet))
                    return 2;
                print(std::cerr, "*** pbx-ping: {}: error: {}: {}\n",
                    uri, code, (reason ? reason : "failure"));
                break;
            }
            std::this_thread::sleep_for(1s);
        }
    }
    catch(const bad_arg& e) {
        exit_code = -1;
        if(!is(opt_quiet))
            die(-1, "Usage: pbx-ping [options] uri\nerror: {}\n", e.what());
    }
    catch(const std::exception& e) {
        exit_code = -1;
        if(!is(opt_quiet))
            die(-1, "error: {}\n", e.what());
    }
    catch(...) {
        exit_code = -1;
        if(!is(opt_quiet))
           die(-1, "error: unknown error\n");
    }
    return exit_code;
}

==> coventry-0.8.1.tar.gz: coventry-0.8.1/utils/pbx-ping.cpp <==
/*
 * Copyright (C) 2022 Tycho Softworks.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <moderncli/args.hpp>
#include <moderncli/print.hpp>
#include <moderncli/strings.hpp>

#include <thread>

#include <unistd.h>
#include <pwd.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <netdb.h>
#include <ifaddrs.h>
#include <netinet/in.h>
#include <net/if.h>
#include <arpa/inet.h>

#include "server.hpp"
#include "ipc.hpp"
#include "exosip.hpp"

using namespace std::literals::string_literals;
using namespace std::chrono_literals;

namespace {
exosip::context_t context("coventry-utils/" PROJECT_VERSION);
std::string uri = "sip:";

bool ipv6 = false;
bool tls = false;
bool tcp = false;
auto port = 5062;
auto ping_duration = 1000;

args::option opt_ipv6('6',      "--ipv6",   "enable ipv6");
args::option opt_config('c',    "--config", "config file path");
args::option helpflag('h',      "--help",   "display this list");
args::option opt_info('i',      "--info",   "display server info");
args::option opt_proxy('p',     "--proxy",  "proxy route", "uri");
args::option opt_quiet('q',     "--quiet",  "quiet one time");
args::option opt_srv('s',       "--srv",    "enable srv lookups");
args::option opt_tcp('t',       "--tcp",    "enable tcp/tls");
args::option althelp('?',       nullptr, nullptr);

auto to_string(const struct sockaddr_storage *addr) {
    const auto sa = reinterpret_cast<const struct sockaddr *>(addr);
    char buf[256];
    switch(sa->sa_family) {
    case AF_INET:
        inet_ntop(AF_INET, &(((struct sockaddr_in *)sa)->sin_addr), buf, sizeof(buf));  // NOLINT
        return "sip:" + std::string(buf) + ":" + std::to_string(ntohs(((struct sockaddr_in *)sa)->sin_port)); // NOLINT
    case AF_INET6:
        inet_ntop(AF_INET6, &(((struct sockaddr_in6 *)sa)->sin6_addr), buf, sizeof(buf));   // NOLINT
        return "sip:[" + std::string(buf) + "]:" + std::to_string(ntohs(((struct sockaddr_in6 *)sa)->sin6_port));   // NOLINT
;
    default:
        return std::string();
    }
}

auto get_ext(std::string_view text) {
    if(text.size() != 2)
        return 0;

    auto v0 = text[0];
    auto v1 = text[1];

    if(v0 < '1' || v0 > '8')
        return 0;

    if(v1 < '0' || v1 > '9')
        return 0;

    return ((v0 - '0') * 10) + v1 - '0';
}

auto from() {
    char host[256];
    std::string user;
    if(gethostname(host, sizeof(host) - 1))
        throw bad_arg("no hostname set");
    host[sizeof(host) - 1] = 0;
    auto pwd = getpwuid(getuid());
    if(pwd && pwd->pw_name && pwd->pw_name[0])
        user = pwd->pw_name;
    else
        throw bad_arg("no user set");
    endpwent();
    return "sip:" + user + "@" + host + ":" + std::to_string(port);
}

void send_ping(const std::string& uri) {
    osip_message_t *msg = nullptr;
    const exosip::guard_t lock(context);
    eXosip_options_build_request(*context, &msg, uri.c_str(), from().c_str(), *opt_proxy);
    eXosip_options_send_request(*context, msg);
}
} // end namespace

auto main(int argc, const char **argv) -> int {
    using namespace exosip;
    using namespace std::chrono;

    int exit_code = 0;
    try {
        auto count = args::parse(argc, argv);
        if(is(helpflag) || is(althelp) || !count) {
            args::help({"pbx-ping [options] uri|ext"}, {
                "pbx-ping " PROJECT_VERSION,
                "David Sugar <tychosoft@gmail.com>",
                "Ping remove sip endpoint"
            });
            return 0;
        }
        if(count > 1)
            throw bad_arg("only use 1 uri");

        auto uri_arg = args::list()[0];
        ipv6 = *opt_ipv6;
        tcp = *opt_tcp;
        auto ext = get_ext(uri_arg);
        std::string reg_uri;

        if(ext) {
            auto rshm = shm_open(SERVER_IPCPATH ".registry", O_RDONLY, 0660);
            if(rshm < 0)
                throw std::runtime_error("server offline");
            struct stat rino{};
            if(fstat(rshm, &rino) < 0)
                throw std::runtime_error("server offline");

            auto rmax = (rino.st_size - sizeof(pbx_sys)) / sizeof(pbx_reg);
            auto rmem = mmap(nullptr, sizeof(pbx_reg) * rmax + sizeof(pbx_sys), PROT_READ, MAP_SHARED, rshm, 0);
            if(rmem == MAP_FAILED)   // NOLINT
                throw std::runtime_error("server failed");

            const auto& reg = static_cast<pbx_reg*>(rmem);
            const auto& entry = reg[ext - 10];
            time_t expires{};

            time(&expires);
            if(!entry.activated || expires > entry.expires) {
                if(!is(opt_quiet))
                    print(stderr, "*** pbx-ping: {}: offline\n", uri_arg);
                ::exit(3);
            }
            reg_uri = to_string(&entry.address);
            uri_arg = reg_uri;
        }

        if(begins_with(uri_arg, "sip:")) {
            tls = false;
            uri = "sip:";
            uri_arg.remove_prefix(4);
        }
        else if(begins_with(uri_arg, "sips:")) {
            tls = true;
            uri = "sips:";
            uri_arg.remove_prefix(5);
        }

        auto pos = uri_arg.find('@');
        auto route = uri;

        uri.append(uri_arg);
        if(pos != std::string::npos)
            route.append(uri_arg.substr(pos));
        else
            route.append(uri_arg);

        if(!is(opt_proxy))
            opt_proxy.set(route.c_str());

        auto type = tcp ? listen_type::tcp : listen_type::udp;
        if(tls)
            type = tcp ? listen_type::tls : listen_type::dtls;

        port = context.find_port(5062, tcp ? IPPROTO_TCP : IPPROTO_UDP);
        context.srv_lookup(is(opt_srv));
        context.set_family(ipv6 ? family_type::ipv6 : family_type::ipv4);
        if(!context.listen("*", type, port))
            throw std::runtime_error("unable to bind to a local port");

        //send_ping(uri); was for exosip2 epoll bug (5.2.0-5.2.1)
        //std::this_thread::sleep_for(std::chrono::milliseconds(20));

        for(;;) {
            send_ping(uri);
            auto start = steady_clock::now();
            auto event = exosip::event_t(context, ping_duration);
            auto reply = duration_cast<milliseconds>(steady_clock::now() - start).count();
            auto response = event.response();
            if(!response) {
                exit_code = 3;
                if(reply < ping_duration)
                    ++exit_code;
                if(is(opt_quiet))
                    return exit_code;
                if(exit_code == 3)
                    print(stderr, "*** pbx-ping: {}: unreachable\n", uri);
                else {
                    print(stderr, "*** pbx-ping: {}: invalid\n", uri);
                    break;
                }
                if(is(opt_info))
                    break;
                continue;
            }

            auto code = response->status_code;
            auto reason = response->reason_phrase;
            if(code == SIP_OK && is(opt_info)) {
                char *text = nullptr;
                size_t size{};
                osip_message_to_str(response, &text, &size);
                if(text) {
                    print("{}\n", text);
                    release(text);
                }
                return 0;
            }
            if(!is(opt_info) && (code >= 200 && code < 500)) {
                if(is(opt_quiet))
                    return 0;
                print("pbx-ping: {}: ok time={}ms\n", uri, reply);
            }
            else {
                exit_code = 2;
                if(is(opt_quiet))
                    return 2;
                print(stderr, "*** pbx-ping: {}: error: {}: {}\n",
                    uri, code, (reason ? reason : "failure"));
                break;
            }
            std::this_thread::sleep_for(1s);
        }
    }
    catch(const bad_arg& e) {
        exit_code = -1;
        if(!is(opt_quiet))
            die(-1, "Usage: pbx-ping [options] uri\nerror: {}\n", e.what());
    }
    catch(const std::exception& e) {
        exit_code = -1;
        if(!is(opt_quiet))
            die(-1, "error: {}\n", e.what());
    }
    catch(...) {
        exit_code = -1;
        if(!is(opt_quiet))
           die(-1, "error: unknown error\n");
    }
    return exit_code;
}
